@@ -47,10 +47,6 @@ function render_plus() {
     return '<i class="fa fa-plus-square" aria-hidden="true"></i>';
 }
 
-function render_grip() {
-    return '<i class="fa fa-grip-horizontal" aria-hidden="true"></i>';
-}
-
 /**
  * check tables to see if unsaved changes
  *
@@ -130,7 +126,7 @@ function checkTables(base, row, debug) {
  */
 function ChildRow(table, config, editor, base, parentid, tablename) {
     var that = this;
-    that.debug = true;
+    that.debug = false;
 
     if (that.debug) {console.log(new Date().toISOString() + ' ChildRow()');}
 
